%==============   Constraints   ====================

%-----Power supplied from power sources must match the power demand----

%Necessary matrix manupliation
fcFuelRefReshaped = reshape(fcFuelEff(1:nFC,:),...
    size(fcFuelEff(1:nFC,:),1),1,1,1,size(fcFuelEff,2));
fcFuelRefReshaped1 = fcFuelRefReshaped.*...
    ones(nFC,nFCellPacks,nO,nT,nFuels);
fcPowerReshaped = fcPowerArray(1:nFC,:).*...
    ones(nFC,nFCellPacks,nO,nT,nFuels).*fcFuelRefReshaped1;
bPowerReshaped = bPowerArray.*ones(nB,nBPacks,nO,nT);
bCapacityReshaped = bCapacityArray.*ones(nB,nBPacks,nO,nT);
dePowerReshaped = deP'.*ones(nDE,nO,nT,nFuelsDE);
%assumed that diesel engine power specicifed by manufacturer is with
%efficiency accounted for

%Defining optimization constraint
powerSuppliedCon = optimconstr(nT,nO);

%create constraints if not running for diesel engines only
if deOnly == 0            
    
for t = 1:nT
    for o = 1:nO
        pfc = sum(sum(sum(lFC(:,:,o,t,:).*fcPowerReshaped(:,:,o,t,:))));
        pb = 0.95*sum(sum(sum(dodB(:,:,o,t).*...
            bCapacityReshaped(:,:,o,t)/...
            (fractionOfTime(t,o)*Dr))));
        pde = deEfficiency*sum(sum(lDE(:,o,t,:).*...
            dePowerReshaped(:,o,t,:)));
        %if electricity-mode or not
        if statesTable.deAllowed(t,o) == 1
            powerSuppliedCon(t,o) = pde + pfc + pb == powerDemand(t,o);
            % pde + pb + pfc
        else
            powerSuppliedCon(t,o) = pfc + pb == powerDemand(t,o);
        end
    end
end

else %deOnly == 1, testing diesel engines only
    
for t = 1:nT
    for o = 1:nO
        pde = deEfficiency*sum(sum(lDE(:,o,t,:).*...
            dePowerReshaped(:,o,t,:)));
        powerSuppliedCon(t,o) = ...
            pde == powerDemand(t,o) + maxPeakShavingP(1,o,t);
    end
end
end

powerSystemProblem.Constraints.powerSupplied = powerSuppliedCon;

%Skip constraints if only testing for diesel engines
if deOnly == 0
%----------Power from fuel cells must be within its limits -------------
%Below upper limit
fcLoadUpperCon = optimconstr(nFC,nFCellPacks,nFuels,nT,nO);

for e = 1:nFC
   for i = 1:nFCellPacks
       for f = 1:nFuels
           for t = 1:nT
               for o = 1:nO
                   fcLoadUpperCon(e,i,f,t,o) = ...
                       lFC(e,i,o,t,f) <= fcTable.fcUL(e)*...
                       fcIofESwitchedOn(e,i,o,t,f);
               end
           end
       end
   end
end

powerSystemProblem.Constraints.fcloadUpper = fcLoadUpperCon;

%Above lower limit
fcLoadLowerCon = optimconstr(nFC,nFCellPacks,nFuels,nT,nO);

for e = 1:nFC
   for i = 1:nFCellPacks
       for f = 1:nFuels
           for t = 1:nT
               for o = 1:nO
                   fcLoadLowerCon(e,i,f,t,o) =...
                       lFC(e,i,o,t,f) >= fcTable.fcLL(e)*...
                       fcIofESwitchedOn(e,i,o,t,f);
               end
           end
       end
   end
end

powerSystemProblem.Constraints.fcloadLower = fcLoadLowerCon;
%----------Ensuring that fuel cells are switched on if used-------------
%using fuel f
fcRunningCon = optimconstr(nFC,nFCellPacks,nFuels,nT,nO);

for e = 1:nFC
    for i = 1:nFCellPacks
        for f = 1:nFuels
            for t = 1:nT
                for o = 1:nO
                    fcRunningCon(e,i,f,t,o) = ...
                        lFC(e,i,o,t,f) <= fcIofESwitchedOn(e,i,o,t,f);
                end
            end
        end
    end
end

powerSystemProblem.Constraints.fcRunning = fcRunningCon;
%------Restriction on max number of fuel cells in use at a time-----
%in combination with fuel f
fcMaxNumberInUseCon = optimconstr(nT,nO);

for t = 1:nT
    for o = 1:nO
        fcMaxNumberInUseCon(t,o) = ...
            sum(sum(sum(fcIofESwitchedOn(:,:,o,t,:)))) <= 1;
    end
end

powerSystemProblem.Constraints.fcMaxNumberInUse = fcMaxNumberInUseCon;
%----------Restricion on use of fuel cells, it must be bought-----------
fcBoughtCon = optimconstr(nFC,nFCellPacks);

for e = 1:nFC
    for i = 1:nFCellPacks
            fcBoughtCon(e,i) = ...
                sum(sum(sum(fcIofESwitchedOn(e,i,:,:,:)))) <= ...
                fcBought(e,i)*nFuels*nO*nT;
    end
end

powerSystemProblem.Constraints.fcBought = fcBoughtCon;
%----------Only one fuel cell combination i of type e-------------------
fcMaxOneIofECon = optimconstr(nFC,1);

for e = 1:nFC
    fcMaxOneIofECon(e) = sum(fcBought(e,:)) <= 1;
end

powerSystemProblem.Constraints.fcMaxOneIofE = fcMaxOneIofECon;
%----------Auxiliary system for fc-pack w/fuel f must be large enough---
fcAuxCon = optimconstr(nT,nO,nFC,nFCellPacks,nFuels);

for t = 1:nT
    for o = 1:nO
        for e = 1:nFC
            for i = 1:nFCellPacks
                for f = 1:nFuels
                    fcAuxCon(t,o,e,i,f) = ...
                        fcIofESwitchedOn(e,i,o,t,f)*...
                        fcPowerArray(e,i)*...
                        Aef(e,f) <= fcAux(e,f);
                end
            end
        end
    end
end

powerSystemProblem.Constraints.fcAux = fcAuxCon;
%----------Enough battery power installed-------------------------------
%Peak shaving
bPowerPSPCon = optimconstr(nB,nT,nO);

for e = 1:nB
    for t = 1:nT
        for o = 1:nO
            bPowerPSPCon(e,t,o) = ...
                sum((bPowerArray(e,:).*bBought(e,:))) >= ...
                sum(sum((bCapacityArray(e,:).*dodB(e,:,o,t))./...
                (fractionOfTime(t,o)*Dr))) +...
                sum(bBought(e,:)*maxPeakShavingP(t,o));
        end
    end
end

powerSystemProblem.Constraints.bPower = bPowerPSPCon;

%Load shedding
bPowerLSPCon = optimconstr(nB,nT,nO);

for e = 1:nB
    for t = 1:nT
        for o = 1:nO
            bPowerLSPCon(e,t,o) = ...
                sum((bPowerArray(e,:).*bBought(e,:))) >= ...
                sum(bBought(e,:)*maxLoadSheddingP(t,o));
        end
    end
end

powerSystemProblem.Constraints.bPowerLS = bPowerLSPCon;
%----------Enough battery capacity installed----------------------------
%Peak shaving
bCapacityPSCon = optimconstr(nT,nO);

for t = 1:nT
    for o = 1:nO
        bCapacityPSCon(t,o) = ...
            ((bCapacityArray(e,:).*bBought(e,:))) >= ...
            maxPeakShavingE(t,o)/(socRef-socMin(e));
    end
end

powerSystemProblem.Constraints.bCapacityPS = bCapacityPSCon;

%Load shedding
bCapacityLSCon = optimconstr(nT,nO);

for t = 1:nT
    for o = 1:nO
        bCapacityLSCon(t,o) = ...
            sum((bCapacityArray(e,:).*bBought(e,:))) >= ...
            maxLoadSheddingE(t,o)/(socRef-socMin(e));
    end
end

powerSystemProblem.Constraints.bCapacityLS = bCapacityLSCon;
%----------If a battery has power, it must be running-------------------
bRunningCon = optimconstr(nB,nBPacks,nT,nO);

for e = 1:nB
    for i = 1:nBPacks
        for t = 1:nT
            for o = 1:nO
                bRunningCon(e,i,t,o) = ...
                    dodB(e,i,o,t) <= bSwitchedOn(e,i,o,t);
            end
        end
    end
end

powerSystemProblem.Constraints.bRunning = bRunningCon;
%----------If a battery is running, it must be bought/selected----------
bBoughtCon = optimconstr(nB,nBPacks);

for e = 1:nB
    for i = 1:nBPacks
        bBoughtCon(e,i) = ...
            sum(sum(bSwitchedOn(e,i,:,:))) <= bBought(e,i)*nO*nT;
    end
end

powerSystemProblem.Constraints.bBought = bBoughtCon;
%----------Max one battery pack i and type e----------------------------
%Needed in order for load-evening to work
bMaxOneInstalledCon = optimconstr(1,1);

bMaxOneInstalledCon(1,1) = sum(sum(bBought(:,:))) <= 1;

powerSystemProblem.Constraints.bMaxOneInstalled = bMaxOneInstalledCon;


end %end deOnly

%----------If a DE has load, it must be switched on + within limit------
%Upper limit
deRunningUpperCon = optimconstr(nDE,nT,nO,nFuelsDE);

for j = 1:nDE
    for t = 1:nT
        for o = 1:nO
            for f = 1:nFuelsDE
                deRunningUpperCon(j,t,o,f) =...
                    lDE(j,o,t,f) <= deSwitchedOn(j,o,t,f)*deUL;
            end
        end
    end
end

powerSystemProblem.Constraints.deRunningUpper = deRunningUpperCon;

%Lower limit
deRunningLowerCon = optimconstr(nDE,nT,nO,nFuelsDE);

for j = 1:nDE
    for t = 1:nT
        for o = 1:nO
            for f = 1:nFuelsDE
                deRunningLowerCon(j,t,o,f) = ...
                    lDE(j,o,t,f) >= deSwitchedOn(j,o,t,f)*deLL;
            end
        end
    end
end

powerSystemProblem.Constraints.deRunningLower = deRunningLowerCon;
%----------Diesel engine j must be bought if it is ever used------------
deBoughtCon = optimconstr(nDE);

for j = 1:nDE
    deBoughtCon(j) = sum(sum(sum(deSwitchedOn(j,:,:,:)))) ...
        <= deBought(j)*nO*nT*nFuelsDE;
end

powerSystemProblem.Constraints.deBought = deBoughtCon;
%----------A diesel engine can only use one fuel at a time--------------
%Should be covered by model

deFuelUsageCon = optimconstr(nT,nO,nDE);

for t = 1:nT
    for o = 1:nO
        for j = 1:nDE
            deFuelUsageCon(t,o,j) = sum(deSwitchedOn(j,o,t,:)) <= 1;
        end
    end
end

powerSystemProblem.Constraints.deFuelUsage = deFuelUsageCon;
%----------Diesel engine fuel compliance--------------------------------
%A fuel for diesel engines can only be used if it is compliant in the 
%operational state

deFuelComplianceCon = optimconstr(nDE,nFuelsDE,nO,nT);

for d = 1:nDE
    for f = 1:nFuelsDE
        for o = 1:nO
            for t = 1:nT
                deFuelComplianceCon(d,f,o,t) = ...
                    deSwitchedOn(d,o,t,f) <= deFCompliance(o,f,t);
            end
        end
    end
end

powerSystemProblem.Constraints.deFuelCompliance = deFuelComplianceCon;
%create constraints if not running for diesel engines only
if deOnly == 0      
%----------Indicator variables for fc-degradation-----------------------
%Upper
fcDegrUpperCon = optimconstr(nFC,nFCellPacks,nO,nT,nFuels);

for e = 1:nFC
    for i = 1:nFCellPacks
        for o = 1:nO
            for t = 1:nT
                for f = 1:nFuels
                    fcDegrUpperCon(e,i,o,t,f) = ...
                        lFC(e,i,o,t,f) - 0.8 <= fcLoadUpper(e,i,o,t,f);
                end
            end
        end
    end
end

powerSystemProblem.Constraints.fcDegrLoadUpper = fcDegrUpperCon;
%----------Enough energy capacity in batteries for roundtrip------------
bEnoughCapacityCon = optimconstr(nT);

capacityArray1 = bCapacityArray.*ones(nB,nBPacks,nO,nT);
bCapConTime = reshape(fractionOfTime',1,1,nO,nT);
bCapConTime1 = bCapConTime.*ones(nB,nBPacks,nO,nT);
socMax = socMax.*ones(nB,nBPacks);
socMin = socMin.*ones(nB,nBPacks);
for t = 1:nT  
    baseEnergyDeliveredInOpstatesInRoundtrip = ...
        dodB(:,:,:,t).*...
        capacityArray1(:,:,:,t)./...
        (bCapConTime1(:,:,:,t)*Dr);
    sumBaseEnergyDelivered = ...
        sum(sum(sum(baseEnergyDeliveredInOpstatesInRoundtrip))); 
    %total base energy delivered in a roundtrip in time period t
    
    
    bEnoughCapacityCon(t) = ...
        sumBaseEnergyDelivered + lostEnergyChargedAtShore(t) <=...
        sum(sum(bCapacityArray(:,:).*...
        bBought(:,:).*(socMax(:,:)-socMin(:,:))));
   
end

powerSystemProblem.Constraints.bEnoughCapacity = bEnoughCapacityCon;
%----------Enough fuel storage for hydrogen-----------------------------
fcFuelStorageCon = optimconstr(nT,nFuels);

t1 = reshape(fractionOfTime',1,1,nO,nT);
t2 = t1.*ones(nFC,nFCellPacks,nO,nT,nFuels);
powers = fcPowerReshaped;

%nFCx1x1x1xnFuels
fcSFCArray1 = reshape(fcTable.sfc(1:nFC,:),nFC,1,1,1,nFuels); 
fcSFCArray2 = fcSFCArray1.*ones(nFC,nFCellPacks,nO,nT,nFuels);



for t = 1:nT
    for f = 1:nFuels
        fuelConsumptionArr = Dr.*t2.*powers.*fcSFCArray2.*lFC;
        fcBaseFuelConsumptionInTons = (...
            1e-6)*sum(sum(sum(fuelConsumptionArr(:,:,:,t,f))));
        
        %storage capacity in tons
        storageCapacityInTons = sum(sum(sTanks(:,f))); 
        %changed ntanks from number of tanks
        %to installed capacity, size of tanks

        fcFuelStorageCon(t,f) = ...
            fcBaseFuelConsumptionInTons  <= storageCapacityInTons;
    end
end

powerSystemProblem.Constraints.fcFuelStorage = fcFuelStorageCon;

end     %end deOnly

