%==================Fuel costs========================

%Define fuel consumption array
fcFuelConsumptionArray = zeros(nFC,nFCellPacks,nO,nT,nFuels);

%Define fuel cost optimization expression
fcFuelCostArray = optimexpr(nFuels,nT);

%Define electricity cost arrays
elConsumptionArray = zeros(nB,nO,nT);
elCostArray = zeros(nB,nO,nT);
elCosts = 0;
deFuelCosts = 0;

%------------------Fuel cells------------------
%Define optimization expressione
powerDeliveredFromFuels = optimexpr(nFuels,nT,nO);
fcEnergyDeliveredFromFuels = optimexpr(nFuels,nT,nO);
fcFuelConsumptionOfFuelsInOandTinTonnes = optimexpr(nFuels,nT,nO);

%Need to expand matrices for matrix multiplication
timeMatrix1 = hYear*statesTable.durTimePeriods(1)*...
    reshape(fractionOfTime',1,nT,nO)
timeMatrix = timeMatrix1.*ones(nFuels,nT,nO)

%Calculate power delivered-->energy delivered-->fuel consumption
for t = 1:nT
    for o = 1:nO
        for f = 1:nFuels
            %power delivered from fuel f in o in t:
            powerDeliveredFromFuels(f,t,o) = ...
                sum(sum(lFC(:,:,o,t,f).*fcPowerArray)); 
            fcEnergyDeliveredFromFuels(f,t,o) = ...
                powerDeliveredFromFuels(f,t,o)*timeMatrix(f,t,o);
            fcFuelConsumptionOfFuelsInOandTinTonnes(f,t,o) = ...
                (1e-6)*fcEnergyDeliveredFromFuels(f,t,o)*fcTable.sfc(1,f);
        end
    end
end

fcFuelConsumptionOfFuelsInTimePeriodsInTonnes = ...
    sum(fcFuelConsumptionOfFuelsInOandTinTonnes,3);

%calculate total fuel cost
for f = 1:nFuels
    for t = 1:nT
        fcFuelCostArray(f,t) = ...
            fcFuelConsumptionOfFuelsInTimePeriodsInTonnes(f,t)*...
            fuelCosts(1,f,t);
    end
end

fcFuelCosts = sum(sum(fcFuelCostArray));


%-------------------Batteries-------------------
%Define optimization expressions
totalEnergyFromBatteries = optimexpr(nT,nO);
elCosts = optimexpr(nT,nO);

%Expand matrix for matrix multiplication
bCapacityReshaped = bCapacityArray.*ones(nB,nBPacks,nO,nT);

%Calculate energy delivered-->electricity cost
for t = 1:nT
     kwhCost = fuelsTable.ePrice(1,1,t);
    for o = 1:nO
        totalEnergyFromBatteries(t,o) = sum(sum(sum(dodB(:,:,o,t).*...
            bCapacityReshaped(:,:,o,t))))*(hYear/Dr);
        %Need to multiply with number of roundtrips (=number 
        ...of times in that op.state) in that
        %time period = hoursinYear/durationOfRoundTrip
        elCosts(t,o) = totalEnergyFromBatteries(t,o)*kwhCost;
    end
end

totalElcosts = sum(sum(elCosts));


%---------------Diesel engines - fuel and emission costs---------------
%Define optimization expression
dePowernDExnOxnTxnFuels = optimexpr(nDE,nO,nT,nFuelsDE);

%Reshape and expand matrices for matrix multiplication
deFracTimeMatrix1 = reshape(fractionOfTime',1,nO,nT);
deFracTimeMatrix = deFracTimeMatrix1.*ones(nDE,nO,nT,nFuelsDE);
deSfocMat = gBase'.*ones(nDE,nO,nT,nFuelsDE);

roundTripsEachYear = hYear/Dr;
durationOfTimePeriodsMat = ...
    reshape(durTimePeriods,1,1,nT).*ones(nDE,nO,nT,nFuelsDE);
roundTripsEachTPeriodMat = ...
    roundTripsEachYear*durationOfTimePeriodsMat;
    
%Calculate power delivered
dePowerMatrix = deP'.*ones(nDE,nO,nT,nFuelsDE);
dePowernDExnOxnTxnFuels = lDE.*dePowerMatrix;
deEnergyDeliveredEachRoundtripMat = ...
    Dr*dePowernDExnOxnTxnFuels.*deFracTimeMatrix;

%Calculate energy delivered
deEnergyTotalMat = ...
    deEnergyDeliveredEachRoundtripMat.*...
    roundTripsEachTPeriodMat;

%Calculate fuel consumption
deFuelConsumptionInTonnesMat = (1e-6)*deEnergyTotalMat.*deSfocMat;
 

deFuelPricesReshaped = reshape(deFCost,1,1,nT,nFuelsDE);
deFuelPricesPermuted = permute(deFuelPricesReshaped,[1,2,4,3]);
deFuelPricesMat = deFuelPricesPermuted.*ones(nDE,nO,nT,nFuelsDE);
    
%Calculate fuel costs
deFuelCostsMat = deFuelConsumptionInTonnesMat.*deFuelPricesMat;

%Total fuel costs
deFuelCosts = sum(sum(sum(sum(deFuelCostsMat))));


%Emissions
%Define optimization expressions
deEmissionsArray = optimexpr(nFuelsDE,nT,nAirEmissions);
deEmissionsCostArray = optimexpr(nFuelsDE,nT,nAirEmissions)

%Calculate energy delivered
deEnergySummedOverEnginesByTimePeriodsAndFuels = sum(deEnergyTotalMat,1);
deEnergySummedOverOpStatesByTimePeriods = ...
    sum(deEnergySummedOverEnginesByTimePeriodsAndFuels,2);

deEnergyByFuelsInTimePeriodsReshaped = ...
    reshape(deEnergySummedOverOpStatesByTimePeriods,nT,nFuelsDE)';

%Calculate costs of emissions
for w = 1:nAirEmissions
    for f = 1:nFuelsDE
        for t = 1:nT
            deEmissionsArray(f,t,w) = ...
                deEnergyByFuelsInTimePeriodsReshaped(f,t)*...
                deEmissionsTable.deEmissions(f,w);
            deEmissionsCostArray(f,t,w) = ...
                (1e-6)*deEmissionsArray(f,t,w)*emissionTaxes(1,w,t);
        end
    end
end

deEmissionsCost = sum(sum(sum(deEmissionsCostArray)));


%==================Emission costs(fuel cells)========
%Emission-table: co2 - ch4 - sox - nox
%Define optimization expressions
fcEmissionsArray = optimexpr(nFuels,nT,nAirEmissions);
fcEmissionsCostArray = optimexpr(nFuels,nT,nAirEmissions);

%fuel cell energy delivered
fcEnergyDeliveredFromFuelsByTimePeriods = ...
    sum(fcEnergyDeliveredFromFuels,3); %defined under fuel costs
                  
%Calculate cost from emissions from fuel cells
for w = 1:nAirEmissions
    for f = 1:nFuels
        for t = 1:nT
            fcEmissionsArray(f,t,w) = ...
                fcEnergyDeliveredFromFuelsByTimePeriods(f,t)*...
                fcEmissionsTable.fcEmissions(f,w,1);
            fcEmissionsCostArray(f,t,w) = ...
                (1e-6)*fcEmissionsArray(f,t,w)*emissionTaxes(1,w,t);
        end
    end
end
fcEmissionCost = sum(sum(sum(fcEmissionsCostArray)));

voyex = totalElcosts +...
    fcFuelCosts +...
    deFuelCosts +...
    fcEmissionCost +...
    deEmissionsCost;

%==================Investment costs==================

%fuel cell stacks
fcModulesInvCosts = 0;
for e = 1:nFC
    for i = 1:nFCellPacks
        fcModulesInvCosts =...
            fcModulesInvCosts + ...
            fcBought(e,i)*fcPowerArray(e,i)*fcNominalPrice(e);
    end
end

%Reformers
fcAuxInvCosts = 0;
for f = 1:nFuels
    for e = 1:nFC
        fcAuxInvCosts = fcAuxInvCosts + Cef(e,f)*fcAux(e,f);
    end
end

%fuel cell tanks
fcFuelTankCosts = 0;
for t = 1:nTankTypes
    for f = 1:nFuels
        fcFuelTankCosts = fcFuelTankCosts +...
            sTanks(t,f)*tankCost(t,f);
    end
end


%Battery stacks
bModulesInvCosts = 0;
bPowerInverterCosts = 0;
bInverterCost = bInverterPrice;
for e = 1:nB
    for i = 1:nBPacks
    bModulesInvCosts = bModulesInvCosts +...
        bBought(e,i)*bCapacityArray(e,i)*bPrice(e);
    bPowerInverterCosts = bPowerInverterCosts +...
        bInverterCost(e)*(bBought(e,i)*bPowerArray(e,i));
    end
end

%diesel engines
deInvestmentCosts = 0;
for j = 1:nDE
    deInvestmentCosts = deInvestmentCosts +...
        deBought(j)*deInvCost*deP(j);
end

capex = ...
    fcModulesInvCosts +...
    fcAuxInvCosts +...
    fcFuelTankCosts +...
    bModulesInvCosts +...
    bPowerInverterCosts +...
    deInvestmentCosts;

%==================Maintenance costs==================


fcMaintenanceCosts = 0;

%Define optimization expression
bMaintenanceCostsArray = optimexpr(nB,1);
deMaintenanceCosts = 0;
for t = 1:nT
    for e = 1:nFC
        for i = 1:nFCellPacks
            fcMaintenanceCosts = ...
                fcMaintenanceCosts +...
                fcAnMaintCost(e)*...
                fcPowerArray(e,i)*...
                fcBought(e,i)*...
                durTimePeriods(1,1);
        end
    end
    
    for j = 1:nDE
        deMaintenanceCosts = ...
            deMaintenanceCosts +...
            deAnnualMaintCost*...
            deP(j)*...
            deBought(j)*...
            durTimePeriods(1,1);
    end
end

bMaintenanceCostsArray = ...
    sum(durTimePeriods,3)*...
    sum(sum(bPowerArray.*bBought)).*bAnnualMaintCost;

bMaintenanceCosts = sum(bMaintenanceCostsArray);

maintenanceCosts = ...
    fcMaintenanceCosts +...
    bMaintenanceCosts +...
    deMaintenanceCosts;

%==================Replacement costs==================

%Fuel cells
%Define optimization expressions
fcReplacementCosts = optimexpr(nFC,nFCellPacks);
fcDegradation = optimexpr(nFC,nFCellPacks);
fcNReplacements = optimexpr(nFC,nFCellPacks);
for e = 1:nFC
    for i = 1:nFCellPacks
        for t = 1:nT
            for o = 1:nO
                for f = 1:nFuels
                    time = hYear*fractionOfTime(t,o)*durTimePeriods(t);
%                         if basevalidation == 1
%                             fcDegradation(e,i) = fcDegradation(e,i) +...
%                         fcHourlyDegradation(e)*...
%                         time*...
%                         (fcIofESwitchedOn(e,i,o,t,f));
%                         else
                        fcDegradation(e,i) = fcDegradation(e,i) +...
                        fcHourlyDegradation(e)*...
                        time*...
                        (fcIofESwitchedOn(e,i,o,t,f) +...
                        fcLoadUpper(e,i,o,t,f)*fcAccDegrFactor);
%                        end
                end
            end
        end
        fcNReplacements(e,i) = fcDegradation(e,i)/...
            (fcAllowedDegradation(e));
        fcReplacementCosts(e,i) =...
            fcNReplacements(e,i)*fcPowerArray(e,i)*fcReplCost(e);
    end
end

totalFCReplCost = sum(sum(fcReplacementCosts));



%Batteries
%Define optimization expressions
lcBatReplCosts = optimexpr(nB,nBPacks);  
nReplacementsFromLC = optimexpr(nB,nBPacks);

lcEnergyThroughput = sum(peakShavingEnergyInEachT);
for e = 1:nB
    for i = 1:nBPacks        
        nReplacementsFromLC(e,i) =...
            bBought(e,i)*lcEnergyThroughput/(availThrput(e)*i);
        lcBatReplCosts(e,i) =...
           bTable.bReplCost(e)*bCapacityArray(e,i)*nReplacementsFromLC(e,i);
    end
end

totalLCBatReplCost = sum(sum(lcBatReplCosts));


bBaseEnergyThroughput = optimexpr(nB,nBPacks);
%Calculate total energy throughput for battery stacks
%Need to multiply with number of roundtrips 
...(=number of times in that op.state) in that
%time period = hoursinYear/durationOfRoundTrip
for e = 1:nB
    for i = 1:nBPacks
        for t = 1:nT
            for o = 1:nO
                bBaseEnergyThroughput(e,i) = ...
                bBaseEnergyThroughput(e,i) +...
                    dodB(e,i,o,t)*...
                    (hYear/Dr)*durTimePeriods(t)*...
                    bTable.bCapacityArray(e,i);
            end
        end
    end
end

%Calculate number of replacements based on energy throughput available
%and energy delivered
numberOfReplacementsOfEachBatteryStackAndTypeFromBaseLoad = ...
    bBaseEnergyThroughput./availThrputArray;

bNominalReplCostArray = bReplCost.*ones(nB,nBPacks);
%Calculate cost of replacement of all battery stacks
costOfReplacementOfEachBatteryStackAndTypeFromBaseLoad = ...
    numberOfReplacementsOfEachBatteryStackAndTypeFromBaseLoad.*...
    bCapacityArray.*...
    bNominalReplCostArray;

%Calculate total cost of replacement of batteries from providing base load
totalBBasePowerReplCost = ...
    sum(sum(costOfReplacementOfEachBatteryStackAndTypeFromBaseLoad,2));


replCost = ...
    totalFCReplCost +...
    totalBBasePowerReplCost +...
    totalLCBatReplCost;

opex = replCost + maintenanceCosts;
costs = voyex + capex + opex;

powerSystemProblem.Objective = costs;



