%Main script - sets up problem and run the various scripts 
%to run the optimization model and process the results



clc 
clear all

%Define variables to control dependency-parameters 
reducedNumberOfStates = 0;      %Reduce computational time by setting to 1
testcase = 1;                   %Choice of scenario
baseValidation = 0;             %For validation on base scenario

switch testcase
    case 1 
        sheet = 1;
        casenr = 'case1newplot';%Filename-string to save results
    case 2 
        sheet = 2;
        casenr = 'case22'; 

end



%Import operational profile
run readOperationalParameters.m


nT = size(powerDemand,1);       %number of time periods and op.states
nO = size(powerDemand,2);

%C
linearised = 1;                 %Runs linearized model variant
batReduced = 0;                 %Consider only one battery type or not
deOnly = 0;                     %Run model for diesel engines only or not
electricityOnly = 0;            %Run model for el only or not

if deOnly ~= 1
deAllowed = 1-elOnly; 

if electricityOnly == 1
    deAllowed = zeros(nT,nO);
end

end

%modify o if reducedNumberOfStates == 0
if reducedNumberOfStates == 1
    fractionOfTime(1,:) = 0.5;
    fractionOfTime(2,:) = 0.5;
    powerDemand = powerDemand*0.5
end

%For filename-generation
Str = datestr(now,'yyyy-mm-dd HH_MM_SS');
filedate = Str([1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]);
resultsDir = '/Users/tormagnusmichaelsen/Dropbox/Skole/Master/Matlab/ProblemBased/new/Results/'

%Import system parameters
run problemSetup.m

%Processing of system parameters
statesTable = table(durTimePeriods,powerDemand,fractionOfTime,deAllowed);

nAirEmissions = size(fcEmissions,2);        %No. of air emission-types
fcEmissionsTable = table(fcEmissions);      %Emissions from fc-fuels
deEmissionsTable = table(deEmissions);      %Emissions from de-fuels


fcTable = table(fcPower,fcPrice,fcReplCost,fcPowerArray,...
    fcEL,fcVoltage,fcAnMaintCost,fcPercentageEOL,...
    fcAllowedDegradation,fcHourlyDegradation,...
    fcUL,fcLL,fcEff,sfc);

fcSystem = table(Aef,Cef,tankCap,tankCost);

fuelsTable = table(fuelCosts,ePrice);

bTable = table(bAh,bV,bWh,bKwh,bP,bPowerArray,bCapacityArray,...
    bPrice,socMin,socMax,bAnnualMaintCost,bReplCost,bEff,bCRate,...
    bCycleLife,availThrput,bInverterPrice)

nDE = size(deP,2);                          %No. of diesel engines
nFuelsDE = size(deFCost,2);                 %No. of de fuels
nFC = 3;                                    %No. of fc-types
nFCellPacks = nFuelCellPacks;               %No. of unique fc-stacks
nB = size(bTable,1);                        %No. of battery types
bTypes = {'Type1'; 'Type2'};                %Naming
%bTypes = bTypes(1);                        %If only considering 1 type
nBPacks = maxBatPacks;                      %No. of battery stacks
nTankTypes = size(fcSystem,1);              %No. of tankt types
nFuels = size(fuelsTable.fuelCosts,2);      %No. of fc-fuels


%Initializing problem
tic
powerSystemProblem = optimproblem;          

%Create variables
display('Creating variables')               
run linCreateVariables.m;

%Create costs
display('Creating costs')
run linCreateCosts.m;   

%Create constraints
display('Creating constraints')
run linCreateConstraints.m;               


%Convert problem to struct to use with Genetic Algorithm
prob = prob2struct(powerSystemProblem);     

%Analyse time spent for problem set-up
toc                                         

%Define options and  solve problem
options = optimoptions(powerSystemProblem,'MaxTime',600);
tic
[sol,fval] = solve(powerSystemProblem,'Options',options)
toc


%Investigate solution

fcRunning = sol.fcIofESwitchedOn
deRunning = sol.deSwitchedOn
fc = sol.lFC
b = sol.dodB
de = sol.lDE
fuelCellBought = sol.fcBought
batteryBought = sol.bBought
dieselEngineBought = sol.deBought

run postProcess.m



