
%Sets up problem

%Used defined input
Dr = 12;                                %Duration of roundtrip in hours
hYear = 8760;                           %Hours in a year

%Define a very large number for costs for 
%components that are uncompatible 
bigPowerM = 99999999999;  
bigFuelM = 99999999999;

durTimePeriods = 10*ones(nT,1);         %Duration of timeperiods in years


%For matrix-consistency
powerDemand = powerDemand(:,1:nO);
fractionOfTime = fractionOfTime(:,1:nO);

bigMP = max(max(powerDemand));
smallM = 0.00001;                   



%================Operational================
%Fuels: h2_700barg, lh2, lng 
%$/tonne
%gaseous hydrogen: https://cafcp.org/content/cost-refill

%----------------Fuel costs-----------------
fcFuels = {'H2700Bar';'LH2';'LNG'};
fuelCosts(:,:,1) = [5544 6160 450];     %$/tonnes
fuelCosts(:,:,2) = 0.9*fuelCosts(:,:,1);

if baseValidation == 1                  %constant price for basevalidation
    fuelCosts(:,:,2) = fuelCosts(:,:,1);
end

%Electricity
kwhCost = 0.064;
ePrice(1,1,1) = kwhCost;
ePrice(1,1,2) = 0.9*kwhCost;

if baseValidation == 1                  %constant price, basevalidation
    ePrice(1,1,2) = kwhCost;
end


%-----Generating load compensation pattern--

%assuming cyclic load variability w/period of 10 seconds (from waves etc)
%= 6 peaks + throughs per minute = 6 peaks and throughs each per minute
%multiply by 60 minutes in an hour, and 12 hours in a roundtrip 
%Assuming little time at full peak power

peakTime = 0.1;                             %duration at peak
if baseValidation == 1                      %to shorten computation time
    peakTime = 0.00001;
end
    
peakTimeTest = 6;                           %test only
peaksAndThroughsPerMinute = 60/10;
pph = peaksAndThroughsPerMinute*60;         %peaks and throughs/hour
pprt = pph*Dr;                              %peaks and throughs/roundtrip
pprt1 = pprt;                               
lcRatioOfPDemand = 0.1/100;
peakShavingP = ...
    lcRatioOfPDemand*mean(mean(powerDemand))*rand(pprt1,nO,nT);
loadSheddingP = ...
    lcRatioOfPDemand*mean(mean(powerDemand))*rand(pprt1,nO,nT);


%-----Calculating battery sizing basis------
maxPeakShavingP = lcRatioOfPDemand.*powerDemand;
maxLoadSheddingP = maxPeakShavingP;

maxPeakShavingE = lcRatioOfPDemand.*powerDemand*peakTime;
maxLoadSheddingE = maxPeakShavingE;

hoursInOPerRoundTripEachT = Dr*fractionOfTime;
numberOfPeaksInEachOPerRoundTripEachT = ...
    hoursInOPerRoundTripEachT*pph;

peakShavingEnergyinEachOPerRoundtripEachT = ...
    numberOfPeaksInEachOPerRoundTripEachT*...
    lcRatioOfPDemand.*powerDemand*peakTime;

peakShavingEnergyPerRoundtripEachT = ...
    sum(peakShavingEnergyinEachOPerRoundtripEachT,2);

nRoundTripsEachT = (hYear/Dr)*durTimePeriods;

peakShavingEnergyInEachT = ...
    peakShavingEnergyPerRoundtripEachT.*nRoundTripsEachT



%================Diesel engines======================


%rated power, sfoc, rpm
run readDieselEngines.m         

%Calculating number of diesel engines to be available of each model
Ym = createMaxEnginesNumber(engines,powerDemand);

%Create engines vector
[deP, gBase] = createEnginesVector(engines,Ym);

%Diesel engines relevant parameters


deEfficiency = 0.95;    

%Upper and lower limit
deUL = 1;                       
deLL = 0.05;                       
deFuelTypes = {'MDO';'ULSFO'};              
deFCost(1,:,1) = [555 800];                 %mdo - ulsfo, $/tonnes
deFCost(1,:,2) = 1.1*(deFCost(1,:,1));      %increase from t1 to t2

if baseValidation == 1                      %for basevalidation, 
    deFCost(1,:,2) = (deFCost(1,:,1));
end

deInvCost = 52;                             %From Bassam
deAnnualMaintCost = 0.1*deInvCost;          

%================Fuel cells and fuels================

%Fuel cells: LT-PEMFC, HT-PEMFC, SOFC
fcTypes = {'LT-PEMFC'; 'HT-PEMFC';'SOFC'};
fcPower = [100 100 100]';
fcNominalPrice = [235 492.5 750]';          %$/kW
fcPrice = fcNominalPrice.*fcPower;
fcAccDegrFactor = 0.3;                      %30% increased degradation

fcReplCost = 0.3.*fcNominalPrice;           %$/kw
fcEL = [20000 25000 30000]';                %Expected life, hours
fcVoltage = [630 700 700]';                 %Rated voltage
fcAnMaintCost = [50 50 50]';                %Annual maintenance cost
fcPercentageEOL = [80 80 80]';              %Performance at EOL

%Calculate allowed degradation
fcAllowedDegradation = ...
    (1/100)*(100.-fcPercentageEOL).*fcVoltage;  

%Calculate hourly degradation in use, v/h
fcHourlyDegradation = ...
    fcAllowedDegradation./fcEL;             
fcUL = [0.95 0.95 0.95]';                   
fcLL = [0.1 0.1 0.1]';                      

%Efficiency of fuel cells using different fuels
%fuels column-wise, fuel cell types row-wise

%EDITED: this is general fuel efficiencies, the same fuel 
%efficiency in all fuel cell types, only related to 
%specific fuel consumption. Efficiency loss is covered
%by nfe in power demand equation
%h2_700barg, lh2, lng
fcEff = [
    0.55 0.55 0.45; 
    0.55 0.55 0.45;
    0.55 0.55 0.45];

%Fuel cell effifiency when used with reformer, 
%assuming 10% loss in 
fcFuelEff = [
    1 1 0.9;
    1 1 0.9;
    1 1 0.9];         

%Energy content in fuels, kJ/kg: h2_700barg, lh2, lng
energyPerKg = [142000 142000 52000];  
kwhPerKg = energyPerKg./3600;

%Specific fuel consumption
%converting to g/kwh
fuelConsumption = (1*1000)./kwhPerKg; 

%Resulting specific fuel consumption
sfc = fuelConsumption./fcEff;


%----Air emissions - co2, ch4, sox, nox-----
%from gilbert et al/nerem, g/kwh
%assessment of full life-cycle air emissions of alternative shipping fuels
emissions.h2 = [0 0 0 0];
emissions.lng = [412 3 0.003 1.17];
emissions.hfo = [541 0.01 3.23 15.8];
emissions.mdo = [524 0.01 0.32 14.8];
emissions.lsfo = [320 0.01 0.15 10];


fcEmissions = [emissions.h2; emissions.h2; emissions.lng];
%Extending array in case of different g/kwh for different fuel cells

fcEmissions =  ones(3,4,size(fcPower,1)).*fcEmissions

deEmissions = [emissions.mdo; emissions.lsfo];
%co2, ch4, sox, nox

%Emission taxes: %$/ton, from 22,27kr/kg (skatteetaten)
%Case 2: ch4, sox, nox converted to equivalent co2
switch testcase
    case 1
        %Time period 1
        emissionTaxes(:,:,1) = [0 0 0 2811];        
        %Time period 2
        emissionTaxes(:,:,2) = [0 0 0 3484];
    case 2
        %Time period 1
        emissionTaxes(:,:,1) = [0 0 0 2811];  
        %Time period 2
        emissionTaxes(:,:,2) = [40 1000 0 11920];       
end

%Modify for base validation case
if baseValidation == 1                      
    emissionTaxes(:,:,2) = [0 0 0 2811];
end


%--------Auxiliary systems---------
%Auxiliary system needed for fuel cell type e (rows) 
%if used with fuel f (columns)
Aef = [
    0 0 1;
    0 0 0;
    0 0 0];

%Cost of auxiliary system/reformer for fuel cells
%assuming same cost for all $/kW
lngAuxLT = 46.62;                               %LT-PEMFC
lngAuxHT = 46.62;                               %HT-PEMFC
lngAuxSOFC = 46.62;                             %SOFC    
methanolAuxLT = 46.62;
methanolAuxHT = 46.62;
methanolAuxSOFC = 46.62;

Cef = [bigFuelM bigFuelM lngAuxLT;
       bigFuelM bigFuelM lngAuxHT;
       bigFuelM bigFuelM lngAuxSOFC];


%--------Fuel storage--------------
%Tank compatibility - using capacity per module of type h
%for fuel f, setting capacity to zero if incompatible
%tank-type row-wise, fuel column-wise


%EDITED: using size of fuel storage directly in constraints instead
%of capacities per tank
h2LiquidTankCap = 1200;                             
lngTankCap = 1200;                                 
h2_700TankCap = 800;                            


%storage cost: $/ton
%333 for light-duty per kg, assuming economy of scale-->10 times less, also 10% less 
%for gas and lng
h2LiquidTankCost = 30000; %300 vs 50     300 per kg = 300000 per ton, assuming 10 times less, then 30000            
h2_700TankCost = 27000;
lngTankCost = 24000;

tankTypes = {'Cryo';'Gas';'Hull'};

tankCap = [0 h2LiquidTankCap lngTankCap;
           h2_700TankCap 0 0;
           0 0 0];
       
tankCost = [bigFuelM h2LiquidTankCost lngTankCost ;
            h2_700TankCost bigFuelM bigFuelM ;
            bigFuelM bigFuelM bigFuelM ]; %per ton
        


%================Batteries================

%Check model variant
if batReduced == 1                              
bAh = [500]; % 700]';                           %Ampere-hours
bV = [600]; % 600]';                            %Voltage
bPrice = [1000];% 150]';                        %Nominal price
bInverterPrice = [40];
socMin = [0.3];% 0.3]';                         %Lower limit
socMax = [0.8];% 0.8]';                         %Upper limit
socRef = 0.6;                                   %Reference soc    
bAnnualMaintCost = [21];% 21]';                 %$/kWh
bReplCost = [27.63]; % 20]';                    %$/kWh
bEff = [0.95];% 0.95]';                         %Efficiency
bCRate = [1]; % 1]';                            %C-rate
bCycleLife = [2000];% 2500]';                   %no. of cycles guaranteed

bWh = bAh.*bV;                                  %Capacity in Wh
bKwh = bWh./1000;                               %kWh
bP = bKwh;                                      %Power, kW
availThrput = bCycleLife.*bKwh;                 %Available kwh per module


else
    bAh = [500 700]';                           
bV = [600 600]';                            
bPrice = [1000 1300]';                        
bInverterPrice = [40 40]';
socMin = [0.3 0.3]';                        
socMax = [0.8 0.8]';
socRef = 0.6;
bAnnualMaintCost = [21 21]';                
bReplCost = [27.63 27.63]';                   
bEff = [0.95 0.95]';                 
bCRate = [1 1]';                     
bCycleLife = [2000 2300]';           

bWh = bAh.*bV;
bKwh = bWh./1000;
bP = bKwh;                           
availThrput = bCycleLife.*bKwh;      

end



%======Defining fuel cell- and battery stacks============

%-------Fuel cell calculations---------------------------
%Calculate largest number of fc packs necessary, from smallest type
%Assuming that the power rated from manufacturer is with efficiency
%taken into account
nFuelCellPacks = ceil(1.5*max(max(powerDemand))/min(fcPower))



%----Battery calculations--------------------------------
%Calculate min number of battery combinations for each battery type
%(from load compensation) - to use as starting point for the lowest
%rated power and capacity from each battery type
lcEnergyMax = max(max(max(maxPeakShavingE,maxLoadSheddingE)));
lcPowerMax = max(max(max(maxPeakShavingP,maxLoadSheddingP)));

minBatPacksE = ceil(lcEnergyMax./bKwh);
minBatPacksP = ceil(lcPowerMax./bP);
minBatPakcsArray = [minBatPacksE minBatPacksP]

%Calculate the minimum number of battery packs of each type
%to supply enough power or energy in load compensation:
%Outputs a vector with minimum number of battery modules
%from each type, the first entry in power vector and
%capacity vector corresponds to n*nominal power and capacity

minBatPacks = max(minBatPakcsArray,[],2)


%Calculate lost energy

lostEnergyPSEachRoundtripEachT = (1-bEff(1,1))*...
    sum(peakShavingEnergyinEachOPerRoundtripEachT,2);
lostEnergyLSEachRoundtripEachT = lostEnergyPSEachRoundtripEachT;

lostEnergyChargedAtShore = ...
    (lostEnergyPSEachRoundtripEachT...
    + lostEnergyLSEachRoundtripEachT);

%reshape for correct dimensions for summation below
lostEnergyChargedAtShore = reshape(lostEnergyChargedAtShore,2,1);

%Calculate largest number of bat packs necessary, from smallest type,
%from energy demand in a round trip + max energy
baseEnergyDemandRoundTrip = sum((Dr*fractionOfTime.*powerDemand),2);

energyDemandRoundTrip = ...
    baseEnergyDemandRoundTrip + ...
    lostEnergyChargedAtShore;

maxEnergyDemandRoundTrip = max(sum(energyDemandRoundTrip,2))
maxBatPacksArray = ceil((maxEnergyDemandRoundTrip+lcEnergyMax)./bKwh)
maxBatPacks = ceil(1.5*max(maxBatPacksArray))


%Need to create arrays with power rating for each type of fuel cell
%and power and capacity rating for eacy type of battery to 
%represent the stacks

%Batteries
bPowerArray = zeros(size(maxBatPacksArray,1),maxBatPacks);
bCapacityArray = zeros(size(maxBatPacksArray,1),maxBatPacks);
for e = 1:size(bPowerArray,1)
    for i = 1:maxBatPacks
        bPowerArray(e,i) = bP(e)*i;
        bCapacityArray(e,i) = bKwh(e)*i;
        availThrputArray(e,i) = availThrput(e)*i;
    end
end

%Fuel cells
fcPowerArray = zeros(size(fcPower,1),nFuelCellPacks);
for e = 1:size(fcPowerArray,1)
    for i = 1:nFuelCellPacks
        fcPowerArray(e,i) = fcPower(e)*i;
    end
end

